package org.xbib.interlibrary.action.resolve;

import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.interlibrary.api.action.ServiceArguments;

public abstract class AbstractResolveService implements ResolveService {

    private final static Logger logger = Logger.getLogger(AbstractResolveService.class.getName());

    protected ServiceArguments arguments;

    private final AtomicBoolean enabled;

    private Throwable lastException;

    public AbstractResolveService(ServiceArguments arguments) {
        this.arguments = arguments;
        this.enabled = new AtomicBoolean(arguments.getServiceSettingsAsBoolean("enabled", true));
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.enabled.set(enabled);
    }

    @Override
    public boolean isEnabled() {
        return this.enabled.get();
    }

    @Override
    public String getName() {
        return arguments.getServiceSettings("name", null);
    }

    @Override
    public void addThrowable(Throwable throwable) {
        logger.log(Level.SEVERE, throwable.getMessage(), throwable);
        this.lastException = throwable;
    }

    @Override
    public Throwable getLastThrowable() {
        return lastException;
    }
}
