package org.xbib.interlibrary.action.resolve;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import org.xbib.interlibrary.api.action.AbstractAction;
import org.xbib.interlibrary.api.action.Action;
import org.xbib.interlibrary.api.action.ActionArguments;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.action.Service;

public class ResolveAction extends AbstractAction<ResolveRequest, ResolveResponse, ResolveResponseBuilder, ResolveService, ResolveServiceProvider> {

    public static final ResolveAction INSTANCE = new ResolveAction();

    private static final Logger logger = Logger.getLogger(ResolveAction.class.getName());

    private List<ResolveService> services = new ArrayList<>();

    public ResolveAction() {
    }

    @Override
    public Action<ResolveRequest, ResolveResponse, ResolveResponseBuilder, ResolveService> getInstance() {
        return INSTANCE;
    }

    @Override
    public void init(ActionArguments arguments) {
        this.services = createServices("action.resolve.service", arguments);
        logger.log(Level.INFO, () -> "services: " + services.size() +
                " (" + services.stream().map(Service::getName).collect(Collectors.joining(",")) + ")");
    }

    @Override
    public List<ResolveService> getServices() {
        return services;
    }

    @Override
    public ResolveResponseBuilder newResponseBuilder(ResolveRequest request) {
        return ResolveResponse.builder().setRequest(request);
    }

    @Override
    public ResolveResponseBuilder newEmptyResponseBuilder(ResolveRequest request) {
        return ResolveResponse.builder().setRequest(request).setStatus("global", Response.Status.EMPTY_RESPONSE);
    }

    @Override
    public ResolveResponseBuilder newErrorResponse(ResolveRequest request, Throwable throwable) {
        logger.log(Level.SEVERE, throwable.getMessage(), throwable);
        return ResolveResponse.builder().setRequest(request).setStatus("global", Response.Status.ERROR);
    }
}
