package org.xbib.interlibrary.action.resolve;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.interlibrary.api.BibliographicDescription;
import org.xbib.interlibrary.api.action.Request;

public class ResolveRequest implements Request {

    private final Collection<Object> services;

    private final ResolveRequester requester;

    private final Integer offset;

    private final Integer size;

    private final Builder builder;

    protected ResolveRequest(Builder builder) {
        this.services = builder.services;
        this.requester = builder.requester;
        this.offset = 0;
        this.size = 1;
        this.builder = builder;
    }

    @Override
    public Collection<Object> getServices() {
        return services;
    }

    @Override
    public ResolveRequester getRequester() {
        return requester;
    }

    @Override
    public Integer getOffset() {
        return offset;
    }

    @Override
    public Integer getSize() {
        return size;
    }

    @Override
    public String toString() {
        return super.toString() + "[bibliographicDescription=" + builder.bibliographicDescription + "]";
    }

    public BibliographicDescription getBibliographicDescription() {
        return builder.bibliographicDescription;
    }

    public Boolean isSourceEnabled() {
        return builder.isSourceEnabled;
    }

    public Boolean isOnlineResourceEnabled() {
        return builder.isOnlineResourceEnabled;
    }

    public Boolean isPrintResourceEnabled() {
        return builder.isPrintResourceEnabled;
    }

    public Map<String, Object> asMap() {
        TinyMap.Builder<String, Object> map = TinyMap.builder();
        if (getRequester() != null) {
            map.put("isil", getRequester().getISIL());
        }
        if (builder.bibliographicDescription != null) {
            map.putIfNotNull("source", builder.bibliographicDescription.getSource());
            map.putIfNotNull("sourceid", builder.bibliographicDescription.getSourceId());
            map.putIfNotNull("formattedsourceid", builder.bibliographicDescription.getFormattedSourceId());
            map.putIfNotNull("identifiers", builder.bibliographicDescription.getIdentifiers());
            map.putIfNotNull("author", builder.bibliographicDescription.getAuthor());
            map.putIfNotNull("title", builder.bibliographicDescription.getTitle());
            map.putIfNotNull("intitle", builder.bibliographicDescription.getInTitle());
            map.putIfNotNull("publisher", builder.bibliographicDescription.getPublisher());
            map.putIfNotNull("year", builder.bibliographicDescription.getYear());
            map.putIfNotNull("volume", builder.bibliographicDescription.getVolume());
            map.putIfNotNull("issue", builder.bibliographicDescription.getIssue());
            map.putIfNotNull("pagination", builder.bibliographicDescription.getPagination());
        }
        return map.build();
    }

    public static Builder builder() {
        return new Builder();
    }

    public static class Builder {

        private Collection<Object> services;

        private ResolveRequester requester;

        private BibliographicDescription bibliographicDescription;

        private Boolean isSourceEnabled;

        private Boolean isOnlineResourceEnabled;

        private Boolean isPrintResourceEnabled;

        private Builder() {
            this.isSourceEnabled = false;
            this.isOnlineResourceEnabled = true;
            this.isPrintResourceEnabled = true;
        }

        @SuppressWarnings("unchecked")
        public Builder setServices(Object services) {
            this.services = services instanceof Collection ? (Collection<Object>) services :
                    services != null ? Collections.singletonList(services) : Collections.emptyList();
            return this;
        }

        public Builder setRequester(ResolveRequester requester) {
            this.requester = requester;
            return this;
        }

        public Builder setBibliographicDescription(BibliographicDescription bibliographicDescription) {
            this.bibliographicDescription = bibliographicDescription;
            return this;
        }

        public Builder setSourceEnabled(Boolean enabled) {
            this.isSourceEnabled = enabled;
            return this;
        }

        public Builder setOnlineResourceEnabled(Boolean enabled) {
            this.isOnlineResourceEnabled = enabled;
            return this;
        }

        public Builder setPrintResourceEnabled(Boolean enabled) {
            this.isPrintResourceEnabled = enabled;
            return this;
        }

        public ResolveRequest build() {
            return new ResolveRequest(this);
        }
    }
}
