package org.xbib.interlibrary.action.resolve;

import java.io.IOException;
import java.io.UncheckedIOException;
import org.xbib.datastructures.api.Builder;
import org.xbib.datastructures.json.tiny.JsonBuilder;
import org.xbib.interlibrary.api.BibliographicDescription;
import org.xbib.interlibrary.api.PackageInfo;
import org.xbib.interlibrary.api.action.AbstractResponse;
import org.xbib.interlibrary.api.util.ExceptionFormatter;

public class ResolveResponse extends AbstractResponse<ResolveRequest> {

    final ResolveServiceResponseBuilder builder;

    public ResolveResponse(ResolveRequest resolveRequest, ResolveServiceResponseBuilder builder) {
        super(resolveRequest, builder.getStatus());
        this.builder = builder;
    }

    public static ResolveResponseBuilder builder() {
        return new ResolveResponseBuilder();
    }

    public BibliographicDescription getBibliographicDescription() {
        return builder.bibliographicDescription;
    }

    public String getService() {
        return builder.service;
    }

    public String asJson() {
        try {
            Builder jsonBuilder = JsonBuilder.builder()
                    .beginMap()
                    .field("version", PackageInfo.VERSION)
                    .field("status", getStatus().name());
                    if (builder.getThrowable() != null) {
                        jsonBuilder.field("exception", ExceptionFormatter.format(builder.getThrowable()));
                    }
                    jsonBuilder.field("service", builder.service);
                    if (builder.bibliographicDescription != null) {
                        jsonBuilder
                                .fieldIfNotNull("source", builder.bibliographicDescription.getSource())
                                .fieldIfNotNull("sourceid", builder.bibliographicDescription.getSourceId())
                                .fieldIfNotNull("formattedsourceid", builder.bibliographicDescription.getFormattedSourceId())
                                .fieldIfNotNull("identifiers", builder.bibliographicDescription.getIdentifiers())
                                .fieldIfNotNull("author", builder.bibliographicDescription.getAuthor())
                                .fieldIfNotNull("title", builder.bibliographicDescription.getTitle())
                                .fieldIfNotNull("intitle", builder.bibliographicDescription.getInTitle())
                                .fieldIfNotNull("publisher", builder.bibliographicDescription.getPublisher())
                                .fieldIfNotNull("year", builder.bibliographicDescription.getYear())
                                .fieldIfNotNull("volume", builder.bibliographicDescription.getVolume())
                                .fieldIfNotNull("issue", builder.bibliographicDescription.getIssue())
                                .fieldIfNotNull("pagination", builder.bibliographicDescription.getPagination());
                    }
                    jsonBuilder.fieldIfNotNull("_source", builder.object)
                            .endMap();
            return jsonBuilder.build();
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }
    }
}
