package org.xbib.interlibrary.action.resolve;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.api.action.ResponseBuilder;

public class ResolveResponseBuilder implements ResponseBuilder<ResolveRequest, ResolveResponse> {

    private static final Logger logger = Logger.getLogger(ResolveResponseBuilder.class.getName());

    private ResolveRequest request;

    private Response.Status status;

    private Throwable throwable;

    protected List<ResolveServiceResponseBuilder> serviceResponseBuilders;

    public ResolveResponseBuilder() {
        this.status = Response.Status.UNDEFINED;
        this.serviceResponseBuilders = Collections.synchronizedList(new ArrayList<>());
    }

    public void add(ResolveServiceResponseBuilder serviceResponseBuilder) {
        serviceResponseBuilders.add(serviceResponseBuilder);
    }

    @Override
    public ResolveResponseBuilder setRequest(ResolveRequest request) {
        this.request = request;
        return this;
    }

    @Override
    public ResolveResponseBuilder setStatus(String key, Response.Status status) {
        this.status = status;
        return this;
    }

    @Override
    public Response.Status getStatus() {
        return status;
    }

    @Override
    public ResolveResponseBuilder setThrowable(Throwable throwable) {
        this.throwable = throwable;
        return this;
    }

    @Override
    public Throwable getThrowable() {
        return throwable;
    }

    @Override
    public ResolveResponseBuilder setThrowable(String key, String message, Throwable throwable) {
        this.throwable = throwable;
        return this;
    }

    @Override
    public ResolveResponse build() {
        return new ResolveResponse(request, getBestServiceResponse());
    }

    private ResolveServiceResponseBuilder getBestServiceResponse() {
        if (serviceResponseBuilders.isEmpty()) {
            serviceResponseBuilders.add(ResolveServiceResponse.builder().setStatus("undefined", Response.Status.UNDEFINED));
        }
        logger.log(Level.FINE, "found " + serviceResponseBuilders.size() + " resolve service responses: " +
                serviceResponseBuilders.stream().map(ResolveServiceResponseBuilder::getStatus).toList());
        // cascade by status priority: OK, RETRY, EMPTY_RESPONSE, NOT_FOUND, ERROR
        Optional<ResolveServiceResponseBuilder> optional = serviceResponseBuilders.stream().filter(s -> s.getStatus() == Response.Status.OK).findFirst();
        if (optional.isPresent()) {
            this.status = optional.get().getStatus();
            return optional.get();
        }
        optional = serviceResponseBuilders.stream().filter(s -> s.getStatus() == Response.Status.RETRY).findFirst();
        if (optional.isPresent()) {
            this.status = optional.get().getStatus();
            return optional.get();
        }
        optional = serviceResponseBuilders.stream().filter(s -> s.getStatus() == Response.Status.EMPTY_RESPONSE).findFirst();
        if (optional.isPresent()) {
            this.status = optional.get().getStatus();
            return optional.get();
        }
        optional = serviceResponseBuilders.stream().filter(s -> s.getStatus() == Response.Status.NOT_FOUND).findFirst();
        if (optional.isPresent()) {
            this.status = optional.get().getStatus();
            return optional.get();
        }
        optional = serviceResponseBuilders.stream().filter(s -> s.getStatus() == Response.Status.ERROR).findFirst();
        if (optional.isPresent()) {
            this.status = optional.get().getStatus();
            return optional.get();
        }
        optional = serviceResponseBuilders.stream().findFirst();
        if (optional.isPresent()) {
            this.status = optional.get().getStatus();
            return optional.get();
        }
        throw new IllegalStateException("unable to return something");
    }
}
